unit Fraction;

interface

uses Grids;

type
  TFraction = class
    Numerator: integer;
    Denumerator: integer;
    constructor Create();
    function StrToFraction(s: string): TFraction;
    function FractionToStr(f: TFraction): string;
    function GCD(a,b: integer): integer;
    function Swap(f: TFraction): TFraction;
    function ChangeSign(f: TFraction): TFraction;
    function AddFraction(f1,f2: TFraction): TFraction;
    function MultipleFraction(f1,f2: TFraction): TFraction;
    function DivFraction(f1,f2: TFraction): TFraction;
  private
  {...}
  public
  {...}
end;

type
  TMatrix = class
    Column: integer;
    Row: integer;
    Fields: array of array of TFraction;
    constructor Create(c,r: integer);
    procedure CopyRow(row: integer; Table: TStringGrid; Matrix: TMatrix);
    procedure CopyFromTable(Table: TStringGrid; Matrix: TMatrix);
    procedure CopyToTable(Table: TStringGrid; Matrix: TMatrix);
    procedure ScalarXVector(scalar: TFraction; row: integer; Matrix: TMatrix);
    procedure AddTwoVectors(row1,row2: integer; Matrix: TMatrix);
    procedure SwapRows(row1,row2: integer; Matrix: TMatrix);
  private
  {...}
  public
  {...}
end;

const fline: char='/';

implementation
uses SysUtils, Math, Slave1;


{Implementation for class TFraction}
constructor TFraction.Create();
begin
  Numerator := 0;
  Denumerator := 1;
end;

function TFraction.StrToFraction(s: string): TFraction;
var
  f: TFraction;
  tmp,num: string;
  sign, start_pos: integer;
  i,j,k: integer;
  denum_present: boolean;
begin
  f := Create();
  SetLength(tmp,10);
  denum_present := False;
  j := 1;

  if(s[1] <> '-') then
    begin sign := 1; start_pos := 1; end
  else
    begin sign := -1; start_pos := 2; end;

  for i:=1 to Length(s) do
  begin
    if(s[i] = fline) then
      denum_present := True;
  end;

  if denum_present then
  begin
    for i:=start_pos to Length(s) do
    begin
      if(s[i]<>fline) then
      begin
        tmp[j] := s[i];
        Inc(j);
      end
      else
      begin
        SetLength(num,(j-1));
        for k:=1 to (j-1) do num[k] := tmp[k];
        f.Numerator := strtoint(num)*sign;
        j := 1;
      end;
    end;
    SetLength(num,(j-1));
    for k:=1 to (j-1) do num[k] := tmp[k];
    f.Denumerator := strtoint(num);
  end
  else
  begin
    for i:=start_pos to Length(s) do
    begin
      tmp[j] := s[i];
      Inc(j);
    end;
    SetLength(num,(j-1));
    for k:=1 to (j-1) do num[k] := tmp[k];
    f.Numerator := strtoint(num)*sign;
    f.Denumerator := 1;
  end;

  Result := f;
end;

function TFraction.FractionToStr(f: TFraction): string;
var
  s: string;
  r: integer;
begin
  r := f.Numerator mod f.Denumerator;
  if r<>0 then
    s := inttostr(f.Numerator) + fline + inttostr(f.Denumerator)
  else
    s := inttostr(f.Numerator div f.Denumerator);

  Result := s;
end;

function TFraction.GCD(a,b: integer): integer;
var
  x,y,z: integer;
begin
  x := Abs(a);
  y := Abs(b);
  if y<>0 then
    while y<>0 do
    begin
      z := x mod y;
      x := y;
      y := z;
    end;
  Result := x;
end;

function TFraction.Swap(f: TFraction): TFraction;
var
  z: TFraction;
  i: integer;
begin
  z := Create();
  i := Sign(f.Numerator);
  if i=0 then
    z := f
  else if i=-1 then
  begin
    z.Numerator := (-1) * f.Denumerator;
    z.Denumerator := Abs(f.Numerator);
  end
  else
  begin
    z.Numerator := f.Denumerator;
    z.Denumerator := f.Numerator;
  end;
  Result := z;
end;

function TFraction.ChangeSign(f: TFraction): TFraction;
var
  z: TFraction;
begin
  z := Create();
  z.Numerator := (-1)*f.Numerator;
  z.Denumerator := f.Denumerator;
  Result := z;
end;

function TFraction.AddFraction(f1,f2: TFraction): TFraction;
var
  z: TFraction;
  i: integer;
begin
  z := Create();
  if (f1.Denumerator <> f2.Denumerator) then
  begin
    z.Numerator := f1.Numerator*f2.Denumerator + f2.Numerator*f1.Denumerator;
    z.Denumerator := f1.Denumerator * f2.Denumerator;
  end
  else
  begin
    z.Numerator := f1.Numerator + f2.Numerator;
    z.Denumerator := f1.Denumerator;
  end;
  i := GCD(z.Numerator, z.Denumerator);
  z.Numerator := z.Numerator div i;
  z.Denumerator := z.Denumerator div i;
  Result := z;
end;

function TFraction.MultipleFraction(f1,f2: TFraction): TFraction;
var
  z: TFraction;
  i: integer;
begin
  z := Create();
  z.Numerator := f1.Numerator * f2.Numerator;
  z.Denumerator := f1.Denumerator * f2.Denumerator;
  i := GCD(z.Numerator,z.Denumerator);
  z.Numerator := z.Numerator div i;
  z.Denumerator := z.Denumerator div i;
  Result := z;
end;

function TFraction.DivFraction(f1,f2: TFraction): TFraction;
var
  z: TFraction;
  i,j,sgn: integer;
begin
  z := Create();
  sgn := 1;
  i := Sign(f1.Numerator);
  j := Sign(f2.Numerator);
  if (i<>j) then sgn := -1;
  z.Numerator := Abs(f1.Numerator*f2.Denumerator);
  z.Denumerator := Abs(f1.Denumerator*f2.Numerator);
  i := GCD(z.Numerator,z.Denumerator);
  z.Numerator := z.Numerator div i;
  z.Denumerator := z.Denumerator div i;
  z.Numerator := z.Numerator * sgn;
  Result := z;
end;

{.......... TMatrix Class Implementation...............}

constructor TMatrix.Create(c,r: integer);
var
  f: TFraction;
  i,j: integer;
begin
  f := TFraction.Create();
  Column := c;
  Row := r;
  SetLength(Fields,Column,Row);
  for i:=0 to (Column-1) do
  begin
    for j:=0 to (Row-1) do
      Fields[i][j] := f;
  end;
end;

procedure TMatrix.CopyRow(row: integer; Table: TStringGrid; Matrix: TMatrix);
var
  f: array of TFraction;
  i: integer;
begin
  SetLength(f,Matrix.Column);
  for i:=0 to (Matrix.Column-1) do
    f[i] := TFraction.Create();
  for i:=0 to (Matrix.Column-1) do
  begin
    f[i].StrToFraction(Table.Cells[i,row]);
    Matrix.Fields[i][row] := f[i];
  end;
end;

procedure TMatrix.CopyToTable(Table: TStringGrid; Matrix: TMatrix);
var
  f: TFraction;
  i,j: integer;
begin
  f := TFraction.Create();
  for i:=0 to (Matrix.Column-1) do
  begin
    for j:=0 to (Matrix.Row-1) do
    begin
      f:= Matrix.Fields[i][j];
      Table.Cells[i,j] := f.FractionToStr(f);
    end;
  end;
  f.Destroy;
end;

procedure TMatrix.CopyFromTable(Table: TStringGrid; Matrix: TMatrix);
var
  i: integer;
begin
  for i:=0 to (Matrix.Row-1) do
    Matrix.CopyRow(i,Table,Matrix);
end;

procedure TMatrix.ScalarXVector(scalar: TFraction; row: integer; Matrix: TMatrix);
var
  tmp: array of TFraction;
  i: integer;
begin
  SetLength(tmp,Matrix.Column);
  for i:=0 to (Matrix.Column-1) do
    tmp[i] := TFraction.Create();

  for i:=0 to (Matrix.Column-1) do
  begin
    tmp[i].MultipleFraction(scalar,Matrix.Fields[i][row]);
    Matrix.Fields[i][row] := tmp[i];
  end;
end;

procedure TMatrix.AddTwoVectors(row1,row2: integer; Matrix: TMatrix);
var
  f: array of TFraction;
  i: integer;
begin
  SetLength(f,Matrix.Column);
  for i:=0 to (Matrix.Column-1) do
    f[i] := TFraction.Create();
  for i:=0 to (Matrix.Column-1) do
  begin
    f[i].AddFraction(Matrix.Fields[i][row1],Matrix.Fields[i][row2]);
    Matrix.Fields[i][row1] := f[i];
  end;
end;

procedure TMatrix.SwapRows(row1,row2: integer; Matrix: TMatrix);
var
  f: array of TFraction;
  i: integer;
begin
  SetLength(f,Matrix.Column);
  for i:=0 to (Matrix.Column-1) do
    f[i] := TFraction.Create();
  for i:=0 to (Matrix.Column-1) do
  begin
    f[i] := Matrix.Fields[i][row1];
    Matrix.Fields[i][row1] := Matrix.Fields[i][row2];
  end;
  for i:=0 to (Matrix.Column-1) do
    Matrix.Fields[i][row2] := f[i];
end;

end.
